import base64
import json
import getpass
import hashlib
from cryptography.fernet import Fernet

from pyfort.secret.secret_sqlite  import * 


# To signup for new user 
def signup(user_name, userpwd):
    user_pwd = hashlib.sha256(userpwd.encode())
    key = Fernet.generate_key()
    userpwd = userpwd.encode()
    data = {
        "user_name" : user_name,
        "user_pwd" : user_pwd.hexdigest(),
        "key": key.decode('utf-8')
    }
    json_obj = json.dumps(data, indent = 3)
    file_name = f'{PATH}/{user_name}.json'
    with open(file_name, 'a+') as file:
        file.write(json_obj)
    create_db(user_name)
    
# Sub menu 
def login(user_name, user_pwd):
    file_name = f'{PATH}/{user_name}.json'
    userpwd = hashlib.sha256(user_pwd.encode())
    try:
        with open (file_name, 'r') as file:
            json_obj = json.load(file)
            if json_obj["user_pwd"] == userpwd.hexdigest():
                print(f'\nUser {user_name} found and logged in')
                print(f'Key for {user_name}: {json_obj["key"]}')
                print("""\n[insert / display / get / logout]
insert into database / display whole database / get selected password and decrypt it / logout of current acount""")
                while True:
                    inp = input('>>')
                    if inp.lower() == 'insert':
                        # Combining key with password
                        key = json_obj["key"]
                        key = key[:len(key)-len(user_pwd)] + user_pwd
                        sha256_key = hashlib.sha256(key.encode())
                        sha256_key = sha256_key.hexdigest()
                        sha256_key = bytes(sha256_key[:32],'utf-8')
                        key = base64.urlsafe_b64encode(sha256_key)
                        
                        WEBSITE_NAME = input('  Enter website name:')
                        WEBSITE_USER_NAME = input(f'  Enter username or mail for {WEBSITE_NAME}:')
                        PASSWORD = getpass.getpass(f'  Enter password for {WEBSITE_USER_NAME}:')
                        PASSWORD = PASSWORD.encode()
                        # Encrypt
                        obj = Fernet(key)
                        PASSWORD = obj.encrypt(PASSWORD).decode('utf-8')
                        insert_into(user_name, WEBSITE_NAME, WEBSITE_USER_NAME, PASSWORD)
                    # For show option    
                    elif inp.lower() == 'get':
                        website_name = input('  Enter Website name :')
                        # Combining key with password
                        key = json_obj["key"]
                        key = key[:len(key)-len(user_pwd)] + user_pwd
                        sha256_key = hashlib.sha256(key.encode())
                        sha256_key = sha256_key.hexdigest()
                        sha256_key = bytes(sha256_key[:32],'utf-8')
                        key = base64.urlsafe_b64encode(sha256_key)
                        obj = Fernet(key)
                        # Retrive and decrypt
                        result = show(user_name, website_name)
                        for _ in result:
                            print('\n','     User name:', _[0])
                            pwd = obj.decrypt(_[1].encode()).decode('utf-8')
                            print('      Password:', pwd)
                    # For display option    
                    elif inp.lower() == 'display' or inp.lower() == 'display all':
                        display_db(user_name)
                    # For logout option    
                    elif inp.lower() == 'logout':
                        del(user_pwd)
                        print(f'Logged Out of  user {user_name}\n')
                        del(user_name)
                        break
    except FileNotFoundError:
        print(f"No such user {user_name} found")
    
# Verify user password 
def verify_password(user_name, userpwd , file_name):
    userpwd = hashlib.sha256(userpwd.encode())
    with open (file_name, 'r') as file:
            json_obj = json.load(file)
            if json_obj["user_pwd"] == userpwd.hexdigest():
                return True
            else:
                return False



import os

from colorama import init
from termcolor import colored, cprint
from pyfort.secret.secret_json import * 


# use Colorama to make Termcolor work on Windows too
init()

# Main menu
def show_menu():
    print("""\n[Login / Signup / Erase]
Login:Already created account / Signup:Create new account / Erase:Delete account!!!""")
    while True:    
        choice = input('>')
        # To login into account
        if choice.lower() == 'login':
            user_name = input(' Enter user name:')
            user_pwd = getpass.getpass(f' Enter password for {user_name}:')
            login(user_name, user_pwd)
            del(user_pwd)
            del(user_name)
                
        # To signup for new user    
        elif choice.lower() == 'signup':
            try:
                print('>Creating new user')
                user_name = input(' Enter user name:')
                loc = f'{PATH}/{user_name}.json'
                # Check if user name is available
                if os.path.exists(loc):
                    print(' User already exist')
                    print(' Try another name')
                else:
                    userpwd = getpass.getpass(f' Enter password for {user_name}:')
                    signup(user_name, userpwd)
                    print(colored('Account created', 'green'))
            except Exception as Error:
                print(colored('>Something went wrong','red'))
                print(Error)
                
        # To delete all user related files and data
        elif choice.lower() == 'erase':
            flag = False
            user_name = input('Enter user name:')
            userpwd = input(f'Enter password for {user_name}:')
            loc = f'{PATH}/{user_name}.json'
            # Check if user exist
            if os.path.exists(loc):
                print(f'>File {user_name}.json exist')
                print('>Verifying password')
                flag = verify_password(user_name, userpwd, loc)
                if flag:
                    print('!!password verified')
                    confirm = input('>>Confirm want to delete account [y/n]:')
                    confirm1 = input('  >>Sure want to delete account [y/n]:')
                    confirm2 = input('    >>For real want to delete account [y/n]:')
                    if confirm == 'y' and confirm1 == 'y' and confirm2 == 'y':
                        #os.remove("demofile.txt")
                        pass    
                else:
                    print('!!password incorrect')
            else:
                print('No such file found')
                            


from pyfort.menu import *


def main():
    try:
        print("""
        `*kMdMkr`        
       l@@#MeMB@@c       
      l@@L     x@@V      
      E@@       #@0      
    `vB@@xxxxxxx@@Bv`      __         _
    V@@@@@@3}K@@@@@@y     / _|___ _ _| |_
    V@@@@@B` `Q@@@@@w    |  _/ _ \ '_|  _| 
    V@@@@@@] x@@@@@@w    |_| \___/_|  \__|  Password Manager
    V@@@@@#_--#@@@@@w    
    !dRRRRRRRRRRRRRd!""")
        
        print(colored( """
Program Created By a-tharva
Link to Original: github.com/a-tharva/
Info: - This is written in Python
      - Still in development""", 'red'))
        show_menu()
        
    except KeyboardInterrupt:
        print('\nExiting...')


import sqlite3
from pyfort.utils.utils import PATH


# Connect to database data.db
connect = sqlite3.connect(f'{PATH}/data.db')
cursor = connect.cursor()

# Create new database
def create_db(user_name):
    create = f'CREATE TABLE IF NOT EXISTS {user_name}(ID INTEGER PRIMARY KEY AUTOINCREMENT, WEBSITE_NAME TEXT , WEBSITE_USER_NAME BLOB, PASSWORD BLOB)'
    cursor.execute(create)
    print(f'Database with username {user_name} created')

# Insert new row in database
def insert_into(user_name, WEBSITE_NAME, WEBSITE_USER_NAME, PASSWORD):
    insert = f"INSERT INTO {user_name}(WEBSITE_NAME, WEBSITE_USER_NAME, PASSWORD) VALUES('{WEBSITE_NAME}','{WEBSITE_USER_NAME}','{PASSWORD}')"
    cursor.execute(insert)
    cursor.execute('COMMIT')
    
# Print selected row
def show(user_name, website_name):
    user_name = user_name
    website_name = website_name
    select = f"SELECT WEBSITE_USER_NAME, PASSWORD FROM {user_name} WHERE WEBSITE_NAME = '{website_name}'"
    print(f'  {website_name}')
    result = cursor.execute(select)
    return result

# Print whole database
def display_db(user_name):
    create_user_table = f'CREATE TABLE IF NOT EXISTS {user_name}(ID INTEGER PRIMARY KEY AUTOINCREMENT, WEBSITE_NAME TEXT , WEBSITE_USER_NAME BLOB, PASSWORD BLOB)'
    cursor.execute(create_user_table)
    display = f'SELECT * FROM {user_name}'
    cursor.execute(display)
    show = cursor.fetchall()
    
    # For printing in table format 
    print('+---+'+'-'*15+'+'+'-'*30+'+'+'-'*100+'+')
    print('|No.|Website name   |Website user name{0[0]:13s}|Password{0[0]:92s}|'.format(['']))
    print('+---+'+'-'*15+'+'+'-'*30+'+'+'-'*100+'+')
    for row in show:
        print('|{0[0]:3d}|{0[1]:15s}|{0[2]:30s}|{0[3]:50s}'.format(row))
    print('+---+'+'-'*15+'+'+'-'*30+'+'+'-'*100+'+')


'''from save_retrive import *

#search_display_database('asd')

result = show('asd', 'EREW')
for _ in result:
    print('\n','User name:',_[0])
    print(' password :',_[1])
'''
import getpass

passwoed = getpass.getpass('password:')
print(passwoed)